#!/bin/bash

SCRIPT_DIR=$(dirname $_)
PATH+=":$SCRIPT_DIR/../src"

ESPEAK_OUT=/tmp/${USER}_espeak_out.txt
CGSTMAD_OUT=/tmp/${USER}_cgstmad.wav

set -e

print_usage() {
  printf "Usage: %s [-q] [-i] [-o] [-v voice] text\n\
  Options:\n\
    -q          Quiet mode.\n\
    -i          Keep the cgstmad's input file (phonemes).\n\
    -o          Keep the cgstmad's output file (sound).\n\
    -v voice    Available voices:\n\
                  pt1 (default)\n\
                  pt2\n\
                  pt3\n" $(basename $0) >&2
  exit 1
}

quiet_option=
keep_input_option=
keep_output_option=
voice_option=pt1

# ----- Parse the options.
while getopts 'qiov:' option
do
  case $option in
  q)
    quiet_option=1
    ;;
  i)
    keep_input_option=1
    ;;
  o)
    keep_output_option=1
    ;;
  v)
    voice_option="$OPTARG"
    [[ "$voice_option" == -* ]] && print_usage
    ;;
  ?)
    print_usage
    ;;
  esac
done
shift $(($OPTIND - 1))

# ----- Get the text.
if [ $# -eq 0 ] 
then
  print_usage
else
  text="$*"
fi

# ----- Synthesize.

if [ "$quiet_option" ]
then
  cgstmad_options=
else
  cgstmad_options='-v'
fi

case "$voice_option" in
pt1 | pt2 | pt3)
  config_dir=$SCRIPT_DIR/../cfg/$voice_option/espeak

  # Begin with a silence and with frequency = 94 Hz.
  echo "_ 200 0 94" >$ESPEAK_OUT

  #echo "$text" | espeak -v mb-br1 -b -s 160 >>$ESPEAK_OUT
  echo "$text" | espeak -v mb-br1 -b >>$ESPEAK_OUT
  cgstmad $cgstmad_options $config_dir $ESPEAK_OUT $CGSTMAD_OUT
  ;;
*)
  printf "Invalid voice: %s\n" "$voice_option" >&2
  print_usage
  ;;
esac

# ----- Show the phonemes.
if [ ! "$quiet_option" ]
then
  cat $ESPEAK_OUT
fi

# ----- Play the speech sound.
aplay_options=
if [ "$quiet_option" ]; then aplay_options='-q'; fi
aplay $aplay_options $CGSTMAD_OUT

# ----- Remove the temporary files.
if [ ! "$keep_input_option" ]
then
  rm -f $ESPEAK_OUT
fi
if [ ! "$keep_output_option" ]
then
  rm -f $CGSTMAD_OUT
fi
