#!/bin/bash

SCRIPT_DIR=$(dirname $_)
PATH+=":$SCRIPT_DIR/../src"

set -e

print_usage() {
  printf "Usage: %s [-q] [-i] [-o] [-v voice] -f file\n\
   or: %s [-q] [-i] [-o] [-v voice] text\n\
  Options:\n\
    -q          Quiet mode.\n\
    -i          Keep the cgstmad's input file (phonemes).\n\
    -o          Keep the cgstmad's output file (sound).\n\
    -v voice    Available voices:\n\
                  en1 (default)\n\
                  en2\n\
                  en3\n\
    -f file     Read the text from the file.\n" $(basename $0) $(basename $0) >&2
  exit 1
}

quiet_option=
keep_input_option=
keep_output_option=
voice_option=en1
file_option=

# ----- Parse the options.
while getopts 'qiov:f:' option
do
  case $option in
  q)
    quiet_option=1
    ;;
  i)
    keep_input_option=1
    ;;
  o)
    keep_output_option=1
    ;;
  v)
    voice_option="$OPTARG"
    [[ "$voice_option" == -* ]] && print_usage
    ;;
  f)
    file_option="$OPTARG"
    [[ "$file_option" == -* ]] && print_usage
    ;;
  ?)
    print_usage
    ;;
  esac
done
shift $(($OPTIND - 1))

# ----- Configure the voice.
voice_name=cgstmad_$voice_option
case "$voice_option" in
en1 | en2 | en3)
  voice_dir="$SCRIPT_DIR/../cfg/$voice_option/festival/english/$voice_name"
  ;;
*)
  printf "Invalid voice: %s\n" "$voice_option" >&2
  print_usage
  ;;
esac

# ----- Get the text.
if [ ! "$file_option" ]
then
  if [ $# -eq 0 ] 
  then
    print_usage
  else
    text="$*"
  fi
fi

# ----- Other options.
if [ ! "$quiet_option" ]
then
  export CGSTMAD_OPTIONS='-v'
fi
if [ "$keep_input_option" ]
then
  export CGSTMAD_KEEP_INPUT=1
fi
if [ "$keep_output_option" ]
then
  export CGSTMAD_KEEP_OUTPUT=1
fi

# ----- Synthesize.
if [ "$file_option" ]
then
  scm="(voice-location \"$voice_name\" \"$voice_dir\" \"Experimental voice.\")"
  scm+="(voice_$voice_name)(tts \"$file_option\" nil)(audio_mode 'close)"

  echo "$scm" | festival --pipe
else
  scm="(voice-location \"$voice_name\" \"$voice_dir\" \"Experimental voice.\")"
  scm+="(voice_$voice_name)(SayText \"$text\")"

  echo "$scm" | festival --pipe
fi
