/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef FORMULA_SYMBOL_H_
#define FORMULA_SYMBOL_H_

#include "main.h"

#include <string>
#include <tr1/array>
#include <tr1/unordered_map>



namespace Cgstmad {

struct FormulaSymbol {
	enum Code {
		SYMB_TRANSITION1,
		SYMB_TRANSITION2,
		SYMB_TRANSITION3,
		SYMB_TRANSITION4,
		SYMB_QSSA1,
		SYMB_QSSA2,
		SYMB_QSSA3,
		SYMB_QSSA4,
		SYMB_QSSB1,
		SYMB_QSSB2,
		SYMB_QSSB3,
		SYMB_QSSB4,
		SYMB_TEMPO12, // between postures 1 and 2
		SYMB_TEMPO23, // between postures 2 and 3
		SYMB_TEMPO34, // between postures 3 and 4
		SYMB_RD,
		SYMB_MARK1,
		SYMB_MARK2,
		SYMB_NULL,
		NUM_SYMBOLS
	};

	typedef std::tr1::unordered_map<std::string, Code> CodeMap;
	CodeMap codeMap; // symbol name -> symbol code

	FormulaSymbol();
};

typedef std::tr1::array<float, FormulaSymbol::NUM_SYMBOLS> FormulaSymbolList; // [symbol code] -> value

} /* namespace Cgstmad */

#endif /* FORMULA_SYMBOL_H_ */
