/***************************************************************************
 *  Copyright 2007 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef INPUT_FILE_H_
#define INPUT_FILE_H_

#include "main.h"

#include <iostream>
#include <list>
#include <string>
#include <vector>



namespace Cgstmad {

class PhonemeRewriter;
class PostureRewriter;

// File charset: ASCII
// Line format:
// phoneme duration_ms [ pitch_pos_percent1 pitch_target_hz1 [ pitch_pos_percent2 pitch_target_hz2 ...]]
class InputFile {
public:
	struct InputCommand;
	typedef std::list<InputCommand> InputCommandList;

	InputFile(const std::string& filePath);

	void rewritePhonemes(const PhonemeRewriter& rewriter);
	void rewritePostures(PostureRewriter& rewriter);
	void removeFirstCmd(int n);

	const InputCommandList& cmdList() const { return cmdList_; }
private:
	InputCommandList cmdList_;

	void read(std::istream& in, const std::string& filePath);
};

struct InputFile::InputCommand {
	struct FrequencyTarget {
		float position; // in percent
		float frequency;
		FrequencyTarget(float position, float frequency)
			: position(position), frequency(frequency) {}
	};
	typedef std::vector<FrequencyTarget> FrequencyTargetList;

	std::string phoneme;
	float duration;
	FrequencyTargetList frequencyTargetList;

	InputCommand();
	InputCommand(const std::string& line, int lineNum, const std::string& filePath);
};

} /* namespace Cgstmad */

#endif /*INPUT_FILE_H_*/
