/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef PARAMETER_H_
#define PARAMETER_H_

#include "main.h"

#include <string>
#include <tr1/array>
#include <tr1/unordered_map>

#include "controller_exception.h"



namespace Cgstmad {
namespace Parameter {

enum Code {
	PARAMETER_GLOT_PITCH,
	PARAMETER_GLOT_VOL,
	PARAMETER_ASP_VOL,
	PARAMETER_FRIC_VOL,
	PARAMETER_FRIC_POS,
	PARAMETER_FRIC_CF,
	PARAMETER_FRIC_BW,
	PARAMETER_R1,
	PARAMETER_R2,
	PARAMETER_R3,
	PARAMETER_R4,
	PARAMETER_R5,
	PARAMETER_R6,
	PARAMETER_R7,
	PARAMETER_R8,
	PARAMETER_VELUM,
	NUM_PARAMETERS
};

struct CodeMap : public std::tr1::unordered_map<std::string, Code> {
	CodeMap();
};

struct Info {
        // Note: For volume parameters, the values are in dB.
	float minimum;
	//float maximum;
	float span; // maximum - minimum

	typedef std::tr1::array<Info, NUM_PARAMETERS> Array;
};

inline
bool
isVolumeParameter(Code parameter)
{
	switch (parameter) {
	case PARAMETER_GLOT_VOL:
	case PARAMETER_ASP_VOL:
	case PARAMETER_FRIC_VOL:
		return true;
	default:
		return false;
	}
}

void
setInfo(Info::Array& parameterInfoArray, const CodeMap& parameterCodeMap, const std::string& name, float min, float max);

template<typename T>
void
setValue(const CodeMap& parameterCodeMap, T& t, const std::string& name, float value) {

	CodeMap::const_iterator iter = parameterCodeMap.find(name);
	if (iter == parameterCodeMap.end()) {
		throw ControllerException(ControllerError("Parameter not found: ") << name);
	}
	t[iter->second] = value;
}

template<typename T>
void
setValue(const CodeMap& parameterCodeMap, T& t, const std::string& name, const std::string& value) {

	CodeMap::const_iterator iter = parameterCodeMap.find(name);
	if (iter == parameterCodeMap.end()) {
		throw ControllerException(ControllerError("Parameter not found: ") << name);
	}
	t[iter->second] = value;
}

} /* namespace Parameter */
} /* namespace Cgstmad */

#endif /*PARAMETER_H_*/
