/***************************************************************************
 *  Copyright 2007 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "phoneme_rewriter.h"



namespace Cgstmad {

PhonemeRewriter::PhonemeRewriter(const std::string& filePath)
		: translationFile_(filePath)
{
}

void
PhonemeRewriter::rewrite(InputFile::InputCommandList& cmdList) const
{
	const float epsilon = 1.0e-6f;

	typedef InputFile::InputCommand::FrequencyTargetList FrequencyList;

	for (InputFile::InputCommandList::iterator iter = cmdList.begin(); iter != cmdList.end(); ++iter) {
		// Convert to the internal phoneme set.
		translate(iter->phoneme);

		const std::string& s = iter->phoneme;
		std::string::size_type p = s.find(',');
		if (p != std::string::npos) {
			// Composite phoneme.

			InputFile::InputCommand cmd1;
			cmd1.phoneme = s.substr(0, p);
			cmd1.duration = iter->duration / 2.0f;
			InputFile::InputCommand cmd2;
			cmd2.phoneme = s.substr(p + 1, s.size() - (p + 1));
			cmd2.duration = cmd1.duration;

			bool firstHalf = false, secondHalf = false;

			float lastPos = 0.0, lastFreq = 0.0;
			const FrequencyList& freqList = iter->frequencyTargetList;
			for (FrequencyList::const_iterator itFreq = freqList.begin(); itFreq != freqList.end(); ++itFreq) {
				if (itFreq->position < 50.0f + epsilon) {
					cmd1.frequencyTargetList.push_back(
						InputFile::InputCommand::FrequencyTarget(itFreq->position * 2.0f, itFreq->frequency));
					lastPos = itFreq->position;
					lastFreq = itFreq->frequency;
					firstHalf = true;
				} else {
					if (!firstHalf) {
						cmd1.frequencyTargetList.push_back(
								InputFile::InputCommand::FrequencyTarget(100.0f, itFreq->frequency));
					}
					if (!secondHalf) {
						if (lastPos < 50.0f - epsilon) {
							float d1 = 50.0f - lastPos, d2 = itFreq->position - 50.0f;
							cmd1.frequencyTargetList.push_back(
								InputFile::InputCommand::FrequencyTarget(100.0f, (d1 * itFreq->frequency + d2 * lastFreq) / (d1 + d2)));
						}
						secondHalf = true;
					}
					cmd2.frequencyTargetList.push_back(
						InputFile::InputCommand::FrequencyTarget((itFreq->position - 50.0f) * 2.0f, itFreq->frequency));
				}
			}

			cmdList.insert(iter, cmd1);
			*iter = cmd2;
		}
	}
}

} /* namespace Cgstmad */
