/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef POSTURE_H_
#define POSTURE_H_

#include "main.h"

#include <string>
#include <vector>
#include <tr1/array>
#include <tr1/memory>
#include <tr1/unordered_map>

#include "category.h"
#include "parameter.h"



namespace Cgstmad {

class Model;

class Posture {
public:
	struct Symbols {
		float duration;
		float transition;
		float qssa;
		float qssb;

		Symbols() : duration(0.0), transition(0.0), qssa(0.0), qssb(0.0) {}
	};

	Posture();
	Posture(const std::string& name);

	bool isMemberOfCategory(int code) const;
	bool isMemberOfCategory(const std::string& categoryName, bool postureNameOnly) const;
	template<typename T> bool isMemberOfCategory(const std::string& categoryName, T match) const;

	const std::string& name() const {
		return name_;
	}
	float getParameterValue(Parameter::Code parameter) const {
		return parameters_[parameter];
	}
	const Symbols& symbols() const {
		return symbols_;
	}
private:
	std::string name_;
	std::tr1::array<float, Parameter::NUM_PARAMETERS> parameters_;
	CategoryList categoryList_;
	Symbols symbols_;

	friend class ConfigFile;
	friend class Model;
};

typedef std::tr1::shared_ptr<Posture> Posture_ptr;
typedef std::tr1::unordered_map<std::string, Posture_ptr> PostureMap; // type of container that manages the Posture instances
typedef std::vector<const Posture*> PostureTmpList;

/*******************************************************************************
 * Obs.: Considers only the name of the posture.
 */
template<typename T>
bool
Posture::isMemberOfCategory(const std::string& categoryName, T match) const
{
	if (match(name_, categoryName)) return true;

//	for (CategoryList::const_iterator iter = categoryList.begin(); iter != categoryList.end(); ++iter) {
//		if (match(iter->name, categoryName)) return true;
//	}

	return false;
}

/*******************************************************************************
 *
 */
inline
void
insert(PostureMap& map, const std::string& key, Posture_ptr value)
{
	typedef PostureMap::iterator MI;
	typedef PostureMap::value_type VT;

	std::pair<MI, bool> res = map.insert(VT(key, value));
	if (!res.second) {
		throw ControllerException(ControllerError("Duplicate posture: ") << key);
	}
}

} /* namespace Cgstmad */

#endif /*POSTURE_H_*/
