/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef SPECIAL_TRANSITION_PROCESSOR_H_
#define SPECIAL_TRANSITION_PROCESSOR_H_

#include "main.h"

#include <vector>

#include "parameter.h"



namespace Cgstmad {

class Model;
class Transition;

/*******************************************************************************
 *
 */
class SpecialTransitionProcessor {
public:
	SpecialTransitionProcessor(const Model& model, Parameter::Code param);
	~SpecialTransitionProcessor();

	void clear();
	void prepare(const Transition& transition);
	float getValue(float time);

	bool empty() const { return pointList_.empty(); }
private:
	struct Point {
		float time;
		float value;
		Point(float time = 0.0, float value = 0.0) : time(time), value(value) {}
	};
	typedef std::vector<Point> PointList;

	const Model& model_;
	const Parameter::Code parameter_;
	PointList pointList_;
	PointList::size_type pointIndex_; // stores the current point list index

	SpecialTransitionProcessor(const SpecialTransitionProcessor&);
	SpecialTransitionProcessor& operator=(const SpecialTransitionProcessor&);

	bool timeIsValid(float time, float lastTime) const;
};



} /* namespace Cgstmad */

#endif /* SPECIAL_TRANSITION_PROCESSOR_H_ */
