/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef TRANSITION_PROCESSOR_H_
#define TRANSITION_PROCESSOR_H_

#include "main.h"

#include "parameter.h"
#include "posture.h"
#include "transition.h"



namespace Cgstmad {

class Model;

/*******************************************************************************
 *
 */
class TransitionProcessor {
public:
	TransitionProcessor(const Model& model, Parameter::Code param);
	~TransitionProcessor();

	void prepare(const Transition& transition, const PostureTmpList& postureList);
	float getValue(size_t basePosture, float time);
private:
	struct Point {
		float time;
		float value;
		Point(float time = 0.0, float value = 0.0) : time(time), value(value) {}
	};
	typedef std::vector<Point> PointList;
	typedef std::vector<PointList> PointListList;
	typedef std::vector<PointList::size_type> PointIndexList;

	const Model& model_;
	const Parameter::Code parameter_;
	Transition::Type type_;
	PointListList pointListList_;
	PointIndexList pointIndexList_; // stores the current point list index

	TransitionProcessor(const TransitionProcessor&);
	TransitionProcessor& operator=(const TransitionProcessor&);

	bool timeIsValid(size_t basePosture, float time, float lastTime) const;
};

} /* namespace Cgstmad */

#endif /* TRANSITION_PROCESSOR_H_ */
