/***************************************************************************
 *  Copyright 2007, 2008, 2009 Marcelo Y. Matuda                           *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "exception.h"

#include <cstring>



namespace Cgstmad {

/*******************************************************************************
 * Constructor.
 */
Error::Error(const char* message) : constructorMessage_(message) {
	if (message != 0) {
		try {
			messageBuffer_ << message;
		} catch (...) {
			// Ignore.
		}
	}
}

/*******************************************************************************
 * Constructor.
 */
Exception::Exception(const char* message) throw()
{
	setMessage(message);
}

/*******************************************************************************
 * Constructor.
 */
Exception::Exception(const Error& error) throw()
{
	try {
		setMessage(error.messageBuffer().str().c_str());
	} catch (...) {
		setMessage(error.constructorMessage());
	}
}

void
Exception::setMessage(const char* message)
{
	if (message == 0) {
		message_[0] = '\0';
		return;
	}
	strncpy(message_, message, MESSAGE_BUFFER_SIZE - 1);
	message_[MESSAGE_BUFFER_SIZE - 1] = '\0';
}

const char*
Exception::what() const throw()
{
	return message_;
}

/*******************************************************************************
 * Constructor.
 */
ProgramArgsException::ProgramArgsException(const char* message) throw() : Exception(message)
{
}

/*******************************************************************************
 * Constructor.
 */
ProgramArgsException::ProgramArgsException(const Error& error) throw() : Exception(error)
{
}

} /* namespace Cgstmad */
