/***************************************************************************
 *  Copyright 2007, 2008, 2009 Marcelo Y. Matuda                           *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef EXCEPTION_H_
#define EXCEPTION_H_

#include "main.h"

#include <exception>
#include <sstream>



namespace Cgstmad {

/*******************************************************************************
 * Class used to build the error message of the exception.
 */
class Error {
public:
	Error(const char* message = 0);

	template<typename T> Error& operator<<(const T& complement) throw();
	const char* constructorMessage() const { return constructorMessage_; }
	const std::ostringstream& messageBuffer() const { return messageBuffer_; }
private:
	const char* constructorMessage_;
	std::ostringstream messageBuffer_;

	Error(const Error&);
	Error& operator=(const Error&);
};

template<typename T>
Error&
Error::operator<<(const T& complement) throw() {
	try {
		messageBuffer_ << complement;
	} catch (...) {
		// Ignore.
	}
	return *this;
}

/*******************************************************************************
 *
 */
class Exception : public std::exception {
public:
	Exception(const char* message) throw();
	Exception(const Error& error) throw();
	virtual ~Exception() throw() {}

	virtual const char* what() const throw();
private:
	enum {
		MESSAGE_BUFFER_SIZE = 2048
	};

	char message_[MESSAGE_BUFFER_SIZE];

	void setMessage(const char* message);
};

/*******************************************************************************
 *
 */
class ProgramArgsException : public Exception {
public:
	ProgramArgsException(const char* message) throw();
	ProgramArgsException(const Error& error) throw();
	virtual ~ProgramArgsException() throw() {}
};

} /* namespace Cgstmad */

#endif /* EXCEPTION_H_ */
