/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "program_configuration.h"

#include "simple_config_file.h"
#include "text.h"



namespace {

const std::string TRM_CONFIG_FILE = "/trm.config";

} /* namespace */

//==============================================================================

namespace Cgstmad {

ProgramConfiguration::ProgramConfiguration(const std::string& configDirPath)
		: outputFileFormat(0)
                , outputRate(0.0f)
                , controlRate(0.0f)
                , volume(0.0)
                , channels(1)
                , balance(0.0)
                , tp(0.0)
                , tnMin(0.0)
                , tnMax(0.0)
                , pitchFactor(0.0)
                , pitchOffset(0.0)
		, glottalNoiseEndLevelA(0.0)
		, glottalNoiseStartLevelB(0.0)
		, glottalNoisePeakLevel(0.0)
		, glottalNoiseMinimumLevel(0.0)
		, glottalPulseFactor(0.0)
                , glottalLeakFactor(0.0)
                , glottalFluctuationCutoff(0.0)
                , glottalFluctuationFactor(0.0)
                , glottalVibratoFrequency(0.0)
                , glottalVibratoFactor(0.0)
                , length(0.0)
                , lossFactor(0.0)
                , nasalRadius()
                , throatCutoff(0.0)
                , throatVol(0.0)
                , throatOffset(0.0)
                , mixOffset(0.0)
                , globalTempo(0.0)
                , radius1Factor(0.0)
                , radius2Factor(0.0)
                , radius3Factor(0.0)
                , radius4Factor(0.0)
                , radius5Factor(0.0)
                , radius6Factor(0.0)
                , radius7Factor(0.0)
                , radius8Factor(0.0)
                , minRadius(0.0)
                , fricationFactor(0.0)
                , fricationFrequencyFactor(0.0)
                , fricationBandwidthFactor(0.0)
                , aspirationFactor(0.0)
		, returnPhaseStartLevel(0.0)
		, returnPhaseDuration(0.0)
		, modulation(false)
		, bypassTube(false)
		, smoothClosure(false)
{
	// Load the values.

	SimpleConfigFile cfgFile(configDirPath + TRM_CONFIG_FILE);
	cfgFile.readInto(outputFileFormat, "output_file_format", 0, 2);
	cfgFile.readInto(outputRate, "output_rate", 11025.0f, 48000.0f);
	cfgFile.readInto(controlRate, "control_rate", 1.0f, 4000.0f);
	cfgFile.readInto(volume, "volume", -60.0f, 0.0f);
	cfgFile.readInto(channels, "channels", 1, 2);
	cfgFile.readInto(balance, "balance", -1.0f, 1.0f);
	cfgFile.readInto(tp, "tp", 5.0f, 50.0f);
	cfgFile.readInto(tnMin, "tn_min", 5.0f, 50.0f);
	cfgFile.readInto(tnMax, "tn_max", 5.0f, 50.0f);
	cfgFile.readInto(pitchFactor, "pitch_factor", 0.1f, 10.0f);
	cfgFile.readInto(pitchOffset, "pitch_offset", -20.0f, 20.0f);
	cfgFile.readInto(glottalNoiseEndLevelA, "glottal_noise_end_level_a", 0.1f, 0.9f);
	cfgFile.readInto(glottalNoiseStartLevelB, "glottal_noise_start_level_b", 0.0f, 0.5f);
	cfgFile.readInto(glottalNoisePeakLevel, "glottal_noise_peak_level", 0.0f, 1.0f);
	cfgFile.readInto(glottalNoiseMinimumLevel, "glottal_noise_minimum_level", 0.0f, 1.0f);
	cfgFile.readInto(glottalPulseFactor, "glottal_pulse_factor", 0.0f, 1.0f);
	cfgFile.readInto(glottalLeakFactor, "glottal_leak_factor", 0.0f, 0.5f);
	cfgFile.readInto(glottalFluctuationCutoff, "glottal_fluctuation_cutoff", 0.5f, 10.0f);
	cfgFile.readInto(glottalFluctuationFactor, "glottal_fluctuation_factor", 0.0f, 10.0f);
	cfgFile.readInto(glottalVibratoFrequency, "glottal_vibrato_frequency", 0.5f, 10.0f);
	cfgFile.readInto(glottalVibratoFactor, "glottal_vibrato_factor", 0.0f, 10.0f);
	cfgFile.readInto(length, "length", 8.0f, 17.5f);
	cfgFile.readInto(lossFactor, "loss_factor", 0.0f, 5.0f);
	nasalRadius[0] = 0.0f;
	for (int i = 1; i < TRM_TOTAL_NASAL_SECTIONS; i++) {
		cfgFile.readInto(nasalRadius[i], std::string("nasal_radius.") + Text::convertToString(i), 0.1f, 3.0f);
	}
	cfgFile.readInto(throatCutoff, "throat_cutoff", 50.0f, 10000.0f);
	cfgFile.readInto(throatVol, "throat_vol", -100.0f, 0.0f);
	cfgFile.readInto(throatOffset, "throat_offset", 1.0f, 10.0f);
	cfgFile.readInto(mixOffset, "mix_offset", -30.0f, 0.0f);
	cfgFile.readInto(globalTempo, "global_tempo", 0.01f, 100.0f);
	cfgFile.readInto(radius1Factor, "radius_factor.1", 0.1f, 10.0f);
	cfgFile.readInto(radius2Factor, "radius_factor.2", 0.1f, 10.0f);
	cfgFile.readInto(radius3Factor, "radius_factor.3", 0.1f, 10.0f);
	cfgFile.readInto(radius4Factor, "radius_factor.4", 0.1f, 10.0f);
	cfgFile.readInto(radius5Factor, "radius_factor.5", 0.1f, 10.0f);
	cfgFile.readInto(radius6Factor, "radius_factor.6", 0.1f, 10.0f);
	cfgFile.readInto(radius7Factor, "radius_factor.7", 0.1f, 10.0f);
	cfgFile.readInto(radius8Factor, "radius_factor.8", 0.1f, 10.0f);
	cfgFile.readInto(minRadius, "min_radius", 0.1f, 1.0f);
	cfgFile.readInto(fricationFactor, "frication_factor", 0.1f, 10.0f);
	cfgFile.readInto(fricationFrequencyFactor, "frication_frequency_factor", 0.1f, 10.0f);
	cfgFile.readInto(fricationBandwidthFactor, "frication_bandwidth_factor", 0.1f, 10.0f);
	cfgFile.readInto(aspirationFactor, "aspiration_factor", 0.1f, 10.0f);
	cfgFile.readInto(returnPhaseStartLevel, "return_phase_start_level", 0.0f, 1.0f);
	cfgFile.readInto(returnPhaseDuration, "return_phase_duration", 0.0f, 50.0f);
	cfgFile.readInto(modulation, "modulation");
	cfgFile.readInto(bypassTube, "bypass_tube");
	cfgFile.readInto(smoothClosure, "smooth_closure");

	// Apply other limits.

	if (tnMax < tnMin) tnMax = tnMin;
}

} /* namespace Cgstmad */
