/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2007   *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard                                                         *
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#ifndef PROGRAM_CONFIGURATION_H_
#define PROGRAM_CONFIGURATION_H_

#include "main.h"

#include <string>
#include <vector>

#include "tube_model_constants.h"



namespace Cgstmad {

struct ProgramConfiguration {
	int outputFileFormat;           // file format (0=AU, 1=AIFF, 2=WAVE)
	float outputRate;               // output sample rate (11025 - 48000 Hz)
	float controlRate;              // 1.0-1000.0 input tables/second (Hz)
	float volume;                   // master volume (-60 - 0 dB)
	int channels;                   // # of sound output channels (1, 2)
	float balance;                  // stereo balance (-1 to +1)
	float tp;                       // % glottal pulse rise time
	float tnMin;                    // % glottal pulse fall time minimum
	float tnMax;                    // % glottal pulse fall time maximum
	float pitchFactor;              // glottal pitch factor (the pitch variations will be multiplied by this factor) (0.1 - 10.0)
	float pitchOffset;              // glottal pitch offset (applied after the pitch_factor) (+/- semitones)
	float glottalNoiseEndLevelA;    // the end level of the first noise pulse (0.1 - 0.9)
	float glottalNoiseStartLevelB;  // the start level of the second noise pulse (0.0 - 0.5)
	float glottalNoisePeakLevel;    // glottal noise peak level (0.0 - 1.0)
	float glottalNoiseMinimumLevel; // glottal noise minimum level (0.0 - 1.0)
	float glottalPulseFactor;       // glottal pulse level (0.0 - 1.0)
	float glottalLeakFactor;        // factor that will determine the leak at the glottal end (0.0 - 0.5)
	float glottalFluctuationCutoff; // glottal source fluctuation low-pass cutoff frequency (Hz)
	float glottalFluctuationFactor; // glottal source fluctuation factor
	float glottalVibratoFrequency;  // glottal source vibrato frequency (Hz)
	float glottalVibratoFactor;     // glottal source vibrato factor
	float length;                   // nominal tube length (6 - 20 cm)
	float lossFactor;               // junction loss factor in (0 - 5 %)
	float nasalRadius[TRM_TOTAL_NASAL_SECTIONS]; // fixed nasal radii (0 - 3 cm)
	float throatCutoff;             // throat lp cutoff (50 - nyquist Hz)
	float throatVol;                // throat volume (-100 - 0 dB)
	float throatOffset;             // the horizontal distance from the throat to the mouth (cm)
	float mixOffset;                // noise crossmix offset (-30 - 0 dB)
	float globalTempo;              // > 1.0: faster
	float radius1Factor;            // the main tube radius 1 will be multiplied by this factor
	float radius2Factor;            // the main tube radius 2 will be multiplied by this factor
	float radius3Factor;            // the main tube radius 3 will be multiplied by this factor
	float radius4Factor;            // the main tube radius 4 will be multiplied by this factor
	float radius5Factor;            // the main tube radius 5 will be multiplied by this factor
	float radius6Factor;            // the main tube radius 6 will be multiplied by this factor
	float radius7Factor;            // the main tube radius 7 will be multiplied by this factor
	float radius8Factor;            // the main tube radius 8 will be multiplied by this factor
	float minRadius;                // minimum tube radius (cm)
	float fricationFactor;          // the frication source will be multiplied by this factor
	float fricationFrequencyFactor; // the frication source center frequency will be multiplied by this factor
	float fricationBandwidthFactor; // the frication source bandwidth will be multiplied by this factor
	float aspirationFactor;         // the aspiration source will be multiplied by this factor
	float returnPhaseStartLevel;    // the return phase will start at this level (0.0 - 1.0)
	float returnPhaseDuration;      // the duration of the return phase (after tp + tn) in % of the glottal pulse (0.0 - 50.0)
	bool modulation;                // pulse modulation of noise (false = OFF, true = ON)
	bool bypassTube;                // if true, output only the glottal wave
	bool smoothClosure;             // if the end of closure is smooth

	ProgramConfiguration(const std::string& configDirPath);
};

} // namespace Cgstmad

#endif /*PROGRAM_CONFIGURATION_H_*/
