/***************************************************************************
 *  Copyright 2007 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef SIMPLE_CONFIG_FILE_H_
#define SIMPLE_CONFIG_FILE_H_

#include "main.h"

#include <map>

#include "exception.h"



namespace Cgstmad {

// File charset: ASCII
// Format: "key = value"
class SimpleConfigFile {
public:
	SimpleConfigFile(const std::string& filePath);

	template<typename T> void readInto(T& var, const std::string& key, const T minValue, const T maxValue, bool mustExist = true) const;
	void readInto(bool& var, const std::string& key, bool mustExist = true) const;
	void readInto(std::string& var, const std::string& key, bool mustExist = true) const;
private:
	typedef std::map<std::string,std::string> Map;
	Map configMap_;
};

template<typename T>
void
SimpleConfigFile::readInto(T& var, const std::string& key, const T minValue, const T maxValue, bool mustExist) const
{
	Map::const_iterator iter = configMap_.find(key);
	if (iter == configMap_.end()) {
		if (mustExist) {
			throw Exception(Error("Key not found in configuration map: ") << key);
		} else {
			return;
		}
	}

	std::istringstream in(iter->second);
	if (!(in >> var)) {
		throw Exception(Error("Value of key: ") << key << " in incorrect format: " << iter->second);
	}

	if (var < minValue) {
		throw Exception(Error("The value of the key: ") << key << " must be >= " << minValue);
	} else if (var > maxValue) {
		throw Exception(Error("The value of the key: ") << key << " must be <= " << maxValue);
	}

	//if (Main::verbose()) cout << "key=[" << key << "] value=[" << var << "]" << endl;
}

} /* namespace Cgstmad */

#endif /*SIMPLE_CONFIG_FILE_H_*/
