/***************************************************************************
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef TIMER_H_
#define TIMER_H_

#include "main.h"

#ifdef HAVE_SYS_TIME_H
# include <sys/time.h>
#endif

#include <ctime>



namespace Cgstmad {

#ifdef HAVE_GETTIMEOFDAY

// Wall clock.
struct Timer {
	double start;
	Timer() : start(0.0) {
		struct timeval tv;
		int rv = gettimeofday(&tv, NULL);
		if (rv != -1) {
			start = tv.tv_sec + tv.tv_usec * 1.0e-6;
		}
	}
	~Timer() {}
	double getTime() {
		struct timeval tv;
		int rv = gettimeofday(&tv, NULL);
		if (rv == -1) {
			return 0.0;
		} else {
			return tv.tv_sec + tv.tv_usec * 1.0e-6 - start;
		}
	}
};

#else /* HAVE_GETTIMEOFDAY */

// Processor time used by the program / low resolution.
struct Timer {
	clock_t start;
	Timer() : start(clock()) { }
	~Timer() {}
	double getTime() {
		clock_t finish = clock();
		return static_cast<double>(finish - start) / CLOCKS_PER_SEC;
	}
};

#endif /* HAVE_GETTIMEOFDAY */

} /* namespace Cgstmad */

#endif /*TIMER_H_*/
