/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef AVERAGE_FILTER_H_
#define AVERAGE_FILTER_H_

#include "cgstmad_trm.h"

#include <cassert>
#include <vector>

#include "math_util.h"



namespace Cgstmad {

/*******************************************************************************
 * Moving average filter.
 */
template<typename F>
class AverageFilter {
public:
	AverageFilter(F sampleRate, F period /* seconds */);

	F filter(F value);
private:
	std::vector<F> buf_;
	typename std::vector<F>::size_type pos_;
	double sum_;
	double invN_;
};



/*******************************************************************************
 * Constructor.
 */
template<typename F>
AverageFilter<F>::AverageFilter(F sampleRate, F period)
		: buf_(MathUtil::roundToInt<typename std::vector<F>::size_type>(sampleRate * period))
		, pos_(buf_.size())
		, sum_(0.0)
		, invN_(1.0 / buf_.size())
{
	assert(!buf_.empty());
}

/*******************************************************************************
 *
 */
template<typename F>
F
AverageFilter<F>::filter(F value) {
	if (++pos_ >= buf_.size()) {
		if (pos_ > buf_.size()) { // first value
			buf_.assign(buf_.size(), value);
			sum_ = value * static_cast<double>(buf_.size());
		}
		pos_ = 0;
	}
	sum_ -= buf_[pos_];
	sum_ += value;
	buf_[pos_] = value;
	return sum_ * invN_;
}

} /* namespace Cgstmad */

#endif /*AVERAGE_FILTER_H_*/
