/***************************************************************************
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef DELAY_FILTER_H_
#define DELAY_FILTER_H_

#include "cgstmad_trm.h"

#include <vector>

#include "math_util.h"



namespace Cgstmad {

/*******************************************************************************
 * Delay.
 */
template<typename F>
class DelayFilter {
public:
	DelayFilter(F sampleRate, F delay /* seconds */);

	F filter(F value);
private:
	std::vector<F> buf_;
	typename std::vector<F>::size_type pos_;
};



/*******************************************************************************
 * Constructor.
 */
template<typename F>
DelayFilter<F>::DelayFilter(F sampleRate, F delay)
		: buf_(MathUtil::roundToInt<typename std::vector<F>::size_type>(sampleRate * delay) + 1)
		, pos_(0)
{
}

template<typename F>
F
DelayFilter<F>::filter(F value)
{
	buf_[pos_] = value;
	if (++pos_ >= buf_.size()) pos_ = 0;
	return buf_[pos_];
}

} /* namespace Cgstmad */

#endif /*DELAY_FILTER_H_*/
