/***************************************************************************
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef ECL_SAMPLE_RATE_CONVERTER_H_
#define ECL_SAMPLE_RATE_CONVERTER_H_

#include "cgstmad_trm.h"

#ifdef HAVE_LIBSAMPLERATE

#include <cassert>
#include <vector>
#include <samplerate.h>



namespace Cgstmad {

/*******************************************************************************
 *
 */
class ECLSampleRateConverter {
public:
	ECLSampleRateConverter(FloatType inputRate, FloatType outputRate);
	~ECLSampleRateConverter();

	void put(FloatType sample);
	void flush();
	void resetOutBuffer() {
		outBufferPos_ = 0;
		outBuffer_.resize(0);
	}

	unsigned int padSize() const {
		return 0;
	}
	FloatType maximumSampleValue() const {
		return maximumSampleValue_;
	}
	size_t inBufferSize() const {
		return INPUT_BUFFER_SIZE;
	}
	const std::vector<float>& outBuffer() const {
		return outBuffer_;
	}
	size_t outBufferPos() const {
		return outBufferPos_;
	}
	void setOutBufferPos(size_t newPos) {
		outBufferPos_ = newPos;
	}
private:
	enum {
		INPUT_BUFFER_SIZE = 64
	};

	SRC_STATE* srcState_;
	size_t outBufferPos_;
	long outBufferBlockSize_;
	FloatType maximumSampleValue_;
	SRC_DATA srcData_;
	std::vector<float> inBuffer_;
	std::vector<float> outBuffer_;

	ECLSampleRateConverter(const ECLSampleRateConverter&);
	ECLSampleRateConverter& operator=(const ECLSampleRateConverter&);

	void initialize(FloatType inputRate, FloatType outputRate);
	void resample(bool endOfInput = false);
};

/*******************************************************************************
 *
 */
inline
void
ECLSampleRateConverter::put(FloatType sample)
{
	inBuffer_.push_back(sample);
	if (inBuffer_.size() == INPUT_BUFFER_SIZE) {
		resample();
	}
}

/*******************************************************************************
 *
 */
inline
void
ECLSampleRateConverter::flush()
{
	assert(inBuffer_.size() < INPUT_BUFFER_SIZE);

	inBuffer_.insert(inBuffer_.end(), INPUT_BUFFER_SIZE - inBuffer_.size(), 0.0f);

	resample(true);
}

} /* namespace Cgstmad */

#endif /*HAVE_LIBSAMPLERATE*/

#endif /*ECL_SAMPLE_RATE_CONVERTER_H_*/
