/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2005,  *
 *            2007                                                         *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard, Gregory John Casamento                                 *
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#include "fixed_wavetable.h"



namespace Cgstmad {

/*******************************************************************************
 * Keeps the value in the interval [0.0, TABLE_LENGTH).
 */
inline
float
FixedWavetable::bound(float value)
{
	if (value >= Wavetable::TABLE_LENGTH) value -= Wavetable::TABLE_LENGTH;
	return value;
}

/*******************************************************************************
 * Get the next sample.
 *
 * input:
 *   frequency
 * output:
 *   pulse
 *   noiseEnvelope
 */
void
FixedWavetable::get(float frequency, float& pulse, float& noiseEnvelope) //  Plain oscillator
{
	//  First increment the table position, depending on frequency
	incrementPosition(frequency);

	//  Find surrounding integer table positions
	const int lowerPosition = static_cast<int>(currentPosition_);
	const int upperPosition = static_cast<int>(bound(lowerPosition + 1));

	//  Return interpolated table value
	const float tempPulse = wavetable_[lowerPosition]
	                        + ( (currentPosition_ - lowerPosition)
	                            * (wavetable_[upperPosition] - wavetable_[lowerPosition]) );
	pulse = tempPulse;
	noiseEnvelope = glottalNoiseEnvelope_.get(tempPulse);
}

/*******************************************************************************
 * Increments the position in the wavetable according to the desired frequency.
 */
void
FixedWavetable::incrementPosition(float frequency)
{
	const float previousPosition = currentPosition_;

	// Increment the table position.
	currentPosition_ = bound(currentPosition_ + (frequency * basicIncrement_));

	if (currentPosition_ < previousPosition) { // new period
		glottalNoiseEnvelope_.startOpenPhase();
	}
}

} /* namespace Cgstmad */
