/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2007   *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard                                                         *
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#ifndef FIXED_WAVETABLE_H_
#define FIXED_WAVETABLE_H_

#include "cgstmad_trm.h"

#include "glottal_noise_envelope.h"
#include "math_util.h"
#include "tube_model_constants.h"
#include "wavetable.h"



namespace Cgstmad {

/*******************************************************************************
 * Glottal wavetable.
 */
class FixedWavetable {
public:
	template <typename Configuration>
	FixedWavetable(float sampleRate, const Configuration& config);

	void requestUpdate(float amplitude);
	void get(float frequency, float& pulse, float& noiseEnvelope);

	template<typename T>
	static void fillWavetable(float tp, float tn,
					bool smoothClosure, float returnPhaseStartLevel, float returnPhaseDuration,
					T& wavetable);

	Wavetable::WavetableArray& wavetable() {
		return wavetable_;
	}
	void setReturnPhaseStartLevel(float returnPhaseStartLevel) {
		returnPhaseStartLevel_ = returnPhaseStartLevel;
	}
private:
	const float basicIncrement_;
	float currentPosition_;
	float returnPhaseStartLevel_;
	Wavetable::WavetableArray wavetable_;
	GlottalNoiseEnvelope glottalNoiseEnvelope_;

	void incrementPosition(float frequency);

	static float bound(float value);
};



/*******************************************************************************
 * Initialize the wavetable with a glottal pulse (fixed tn).
 */
template<typename T>
void
FixedWavetable::fillWavetable(float tp, float tn,
				bool smoothClosure, float returnPhaseStartLevel, float returnPhaseDuration,
				T& wavetable)
{
	const int tableDiv1 = MathUtil::roundToInt<int>(Wavetable::TABLE_LENGTH * (tp / 100.0f));
	const int tableDiv2 = MathUtil::roundToInt<int>(Wavetable::TABLE_LENGTH * ((tp + tn) / 100.0f));
	const int tableDiv3 = std::min(
					static_cast<int>(Wavetable::TABLE_LENGTH),
					MathUtil::roundToInt<int>(Wavetable::TABLE_LENGTH * ((tp + tn + returnPhaseDuration) / 100.0f)));

	Wavetable::fillWavetable(tableDiv1, tableDiv2, tableDiv2, tableDiv3,
					smoothClosure, returnPhaseStartLevel, returnPhaseDuration,
					wavetable, false);
}

/*******************************************************************************
 * Constructor.
 */
template <typename Configuration>
FixedWavetable::FixedWavetable(float sampleRate, const Configuration& config)
		: basicIncrement_(Wavetable::TABLE_LENGTH / sampleRate)
		, currentPosition_(Wavetable::TABLE_LENGTH)
		, returnPhaseStartLevel_(config.returnPhaseStartLevel)
		, glottalNoiseEnvelope_(config.glottalNoiseEndLevelA, config.glottalNoiseStartLevelB)
{
	fillWavetable(config.tp, config.tn, config.smoothClosure,
			config.returnPhaseStartLevel, config.returnPhaseDuration, wavetable_);
}

/*******************************************************************************
 * Requests an update.
 */
inline
void
FixedWavetable::requestUpdate(float /*amplitude*/)
{
	// Empty.
}

} /* namespace Cgstmad */

#endif /* FIXED_WAVETABLE_H_ */
