/***************************************************************************
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef FLUCTUATION_FILTER_H_
#define FLUCTUATION_FILTER_H_

#include "cgstmad_trm.h"

#include "noise_source.h"
#include "low_pass_filter2.h"



namespace Cgstmad {

/*******************************************************************************
 *
 */
template<typename F>
class FluctuationFilter {
public:
	FluctuationFilter(F sampleRate, F lowPassCutoff);

	F filter(F value, F fluctuationFactor);
private:
	NoiseSource noiseSource_;
	LowPassFilter2<F> lowPassFilter_;
};



/*******************************************************************************
 * Constructor.
 */
template<typename F>
FluctuationFilter<F>::FluctuationFilter(F sampleRate, F lowPassCutoff)
		: noiseSource_(868148613L)
{
	lowPassFilter_.initializeCoefficients(sampleRate, lowPassCutoff);
}

/*******************************************************************************
 *
 */
template<typename F>
F
FluctuationFilter<F>::filter(F value, F fluctuationFactor)
{
	if (fluctuationFactor == 0.0) return value;

	const F factor = 1.0f + fluctuationFactor * lowPassFilter_.filter(noiseSource_.get());
	return factor > 0.0 ? value * factor : 0.0;
}

} /* namespace Cgstmad */

#endif /*FLUCTUATION_FILTER_H_*/
