/***************************************************************************
 *  Copyright 2009 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "glottal_noise_envelope.h"

#include <cassert>



namespace Cgstmad {

/*******************************************************************************
 * Constructor.
 */
GlottalNoiseEnvelope::GlottalNoiseEnvelope(float firstPulseEndLevel, float secondPulseStartLevel)
		: firstPulseEndLevel_(firstPulseEndLevel)
		, secondPulseStartLevel_(secondPulseStartLevel)
		, firstPulseMiddleLevel_(firstPulseEndLevel * 0.5f)
		, phase_(PHASE_OFF)
		, previousPulseValue_(0.0)
		, position_(0)
		, middlePosition_(0)
		, stopPosition_(0)
{
	assert(firstPulseEndLevel_ >= 0.1f && firstPulseEndLevel_ <= 0.9f);
	assert(secondPulseStartLevel_ >= 0.0f && secondPulseStartLevel_ <= 1.0f);
}

/*******************************************************************************
 *
 */
void
GlottalNoiseEnvelope::startOpenPhase()
{
	phase_ = PHASE_OPEN;
	stopPosition_ = middlePosition_ = position_ = 0;
}

/*******************************************************************************
 *
 */
float
GlottalNoiseEnvelope::get(float pulse)
{
	if (phase_ == PHASE_OFF
	    && previousPulseValue_ > secondPulseStartLevel_
	    && pulse <= secondPulseStartLevel_) {
		phase_ = PHASE_CLOSE;
		position_ = 0;
	}

	float value = 0.0;
	switch (phase_) {
	case PHASE_OFF:
		break;
	case PHASE_OPEN:
		if (pulse < firstPulseMiddleLevel_) {
			value = pulse / firstPulseMiddleLevel_;
		} else if (pulse < firstPulseEndLevel_) {
			value = (firstPulseEndLevel_ - pulse) / firstPulseMiddleLevel_;
		} else {
			stopPosition_ = position_;
			middlePosition_ = stopPosition_ >> 1;
			phase_ = PHASE_OFF;
		}
		break;
	case PHASE_CLOSE:
		if (middlePosition_ > 0 && position_ < middlePosition_) {
			value = position_ / static_cast<float>(middlePosition_);
		} else if (stopPosition_ > middlePosition_ && position_ < stopPosition_) {
			value = (stopPosition_ - position_) / static_cast<float>(stopPosition_ - middlePosition_);
		} else {
			phase_ = PHASE_OFF;
		}
		break;
	}

	previousPulseValue_ = pulse;
	++position_;
	return value;
}

} /* namespace Cgstmad */
