/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2007   *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard                                                         *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#ifndef RING_BUFFER_H_
#define RING_BUFFER_H_

#include "cgstmad_trm.h"

#include <iostream>
#include <tr1/array>



namespace Cgstmad {

class SampleRateConverter;

struct RingBuffer {
public:
	enum {
		BUFFER_SIZE = 64
	};

	int padSize;
	int fillSize; // Derived from BUFFER_SIZE and padSize. Remains constant.
	int fillPtr;
	int emptyPtr;
	int fillCounter;
	SampleRateConverter& sampleRateConverter;
	std::tr1::array<FloatType,BUFFER_SIZE> buffer;

	RingBuffer(SampleRateConverter& sampleRateConverter);

	void dataFill(FloatType data);
	void flushBuffer();

	static void incrementIndex(int& index);
	static void decrementIndex(int& index);
private:
	RingBuffer(const RingBuffer&);
	RingBuffer& operator=(const RingBuffer&);

	void dataEmpty();
	void increment();
	//void decrement();
};

inline
void
RingBuffer::incrementIndex(int& index)
{
	if (++index >= BUFFER_SIZE) index -= BUFFER_SIZE;
}

inline
void
RingBuffer::decrementIndex(int& index)
{
	if (--index < 0) index += BUFFER_SIZE;
}

inline
void
RingBuffer::increment()
{
	if (++fillPtr >= BUFFER_SIZE) fillPtr -= BUFFER_SIZE;
}

//inline
//void
//RingBuffer::decrement()
//{
//	if (--fillPtr < 0) fillPtr += BUFFER_SIZE;
//}

/*******************************************************************************
 * Fills the ring buffer with a single sample, increments the counters and
 * pointers, and empties the buffer when full.
 */
inline
void
RingBuffer::dataFill(FloatType data)
{
	buffer[fillPtr] = data;

	/*  INCREMENT THE FILL POINTER, MODULO THE BUFFER SIZE  */
	increment();

	/*  INCREMENT THE COUNTER, AND EMPTY THE BUFFER IF FULL  */
	if (++fillCounter >= fillSize) {
		dataEmpty();

		/* RESET THE FILL COUNTER  */
		fillCounter = 0;
	}
}

/*******************************************************************************
 * Pads the buffer with zero samples, and flushes it by converting the
 * remaining samples.
 */
inline
void
RingBuffer::flushBuffer()
{
	/*  PAD END OF RING BUFFER WITH ZEROS  */
	for (int i = 0; i < (padSize * 2); ++i) dataFill(0.0);

	/*  FLUSH UP TO FILL POINTER - PADSIZE  */
	dataEmpty();
}

} /* namespace Cgstmad */

#endif /*RING_BUFFER_H_*/
