/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2007   *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard                                                         *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#ifndef SAMPLE_RATE_CONVERTER_H_
#define SAMPLE_RATE_CONVERTER_H_

#include "cgstmad_trm.h"

#include <tr1/array>
#include <memory>
#include <vector>

#include "ring_buffer.h"



#define ZERO_CROSSINGS            13                 /*  SRC CUTOFF FRQ      */
#define LP_CUTOFF                 (11.0/13.0)        /*  (0.846 OF NYQUIST)  */

#define N_BITS                    16
#define L_BITS                    8
#define L_RANGE                   256                  /*  must be 2^L_BITS  */
#define M_BITS                    8
#define M_RANGE                   256                  /*  must be 2^M_BITS  */
#define FRACTION_BITS             (L_BITS + M_BITS)
#define FRACTION_RANGE            65536         /*  must be 2^FRACTION_BITS  */
#define FILTER_LENGTH             (ZERO_CROSSINGS * L_RANGE)
#define FILTER_LIMIT              (FILTER_LENGTH - 1)

#define N_MASK                    0xFFFF0000
#define L_MASK                    0x0000FF00
#define M_MASK                    0x000000FF
#define FRACTION_MASK             0x0000FFFF

#define nValue(x)                 (((x) & N_MASK) >> FRACTION_BITS)
#define lValue(x)                 (((x) & L_MASK) >> M_BITS)
#define mValue(x)                 ((x) & M_MASK)
#define fractionValue(x)          ((x) & FRACTION_MASK)



namespace Cgstmad {

/*******************************************************************************
 *
 */
class SampleRateConverter {
public:
	SampleRateConverter(FloatType inputRate, FloatType outputRate);

	void put(FloatType sample);
	void flush();
	void resetOutBuffer() {
		outBufferPos_ = 0;
		outBuffer_.resize(0);
	}

	unsigned int padSize() const {
		return padSize_;
	}
	FloatType maximumSampleValue() const {
		return maximumSampleValue_;
	}
	size_t inBufferSize() const {
		return ringBuffer_->buffer.size();
	}
	const std::vector<float>& outBuffer() const {
		return outBuffer_;
	}
	size_t outBufferPos() const {
		return outBufferPos_;
	}
	void setOutBufferPos(size_t newPos) {
		outBufferPos_ = newPos;
	}
private:
	enum {
		OUT_BUFFER_INITIAL_SIZE = 4096
	};

	unsigned int timeRegisterIncrement_, filterIncrement_, phaseIncrement_;
	unsigned int timeRegister_;
	unsigned int padSize_;
	size_t outBufferPos_;
	FloatType sampleRateRatio_;
	FloatType maximumSampleValue_;
	std::tr1::array<FloatType,FILTER_LENGTH> h_, deltaH_;
	std::vector<float> outBuffer_;
	std::auto_ptr<RingBuffer> ringBuffer_;

	SampleRateConverter(const SampleRateConverter&);
	SampleRateConverter& operator=(const SampleRateConverter&);

	void initialize(FloatType inputRate, FloatType outputRate);
	void initializeFilter();
	void resample();

	friend struct RingBuffer;
};

inline
void
SampleRateConverter::flush()
{
	ringBuffer_->flushBuffer();
}

inline
void
SampleRateConverter::put(FloatType sample)
{
	ringBuffer_->dataFill(sample);
}

} /* namespace Cgstmad */

#endif /*SAMPLE_RATE_CONVERTER_H_*/
