/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2005,  *
 *            2007                                                         *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard, Gregory John Casamento                                 *
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#include "sine_wavetable.h"

#include <cmath>

#include "math_util.h"



namespace Cgstmad {

/*******************************************************************************
 * Constructor.
 */
SineWavetable::SineWavetable(float sampleRate, float frequency, float amplitude)
		: increment_(frequency * TABLE_LENGTH / sampleRate)
		, currentPosition_(TABLE_LENGTH)
{
	for (int i = 0; i < TABLE_LENGTH; ++i) {
		wavetable_[i] = amplitude * std::sin(static_cast<float>(TWO_PI) * (static_cast<float>(i) / TABLE_LENGTH));
	}
}

/*******************************************************************************
 * Destructor.
 */
SineWavetable::~SineWavetable()
{
}

/*******************************************************************************
 * Keeps the value in the interval [0.0, TABLE_LENGTH).
 */
inline
float
SineWavetable::bound(float value)
{
	if (value >= TABLE_LENGTH) value -= TABLE_LENGTH;
	return value;
}

/*******************************************************************************
 * Gets the current value.
 */
float
SineWavetable::get()
{
	// Find surrounding integer table positions.
	const int lowerPosition = static_cast<int>(currentPosition_);
	const int upperPosition = static_cast<int>(bound(lowerPosition + 1));

	// Return interpolated table value.
	return (wavetable_[lowerPosition] +
			((currentPosition_ - lowerPosition) *
					(wavetable_[upperPosition] - wavetable_[lowerPosition])));
}

/*******************************************************************************
 * Updates the current position.
 */
void
SineWavetable::nextPosition()
{
	// Increment the table position.
	currentPosition_ = bound(currentPosition_ + increment_);
}

} /* namespace Cgstmad */
