/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2007   *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard                                                         *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#ifndef TUBE_MODEL_CONSTANTS_H_
#define TUBE_MODEL_CONSTANTS_H_

/*******************************************************************************
 *
 *                                   VELUM
 *                         nasal    -------------------------------------------
 *                             _____| N1  | N2  | N3  | N4  | N5  | N6  | N7  |
 *                            |     -------------------------------------------
 *                            |           |     |     |     |     |     |
 *                            |          NC1   NC2   NC3   NC4   NC5   NC6
 *    oropharynx              |
 *    -------------------------------------------------------------
 *    | S1  | S2  | S3  | S4  | S5  | S6  | S7  | S8  | S9  | S10 |
 *    -------------------------------------------------------------
 *          |     |     |     |     |     |     |     |     |
 *         C1    C2    C3     |    C4     |    C5    C6    C7
 *      R1    R2  | R3  | R4  | R4  | R5  | R5  | R6  | R7  | R8
 *               FC1   FC2   FC3   FC4   FC5   FC6   FC7   FC8
 *
 */

/*  OROPHARYNX REGIONS  */
#define TRM_R1                        0      /*  S1  */
#define TRM_R2                        1      /*  S2  */
#define TRM_R3                        2      /*  S3  */
#define TRM_R4                        3      /*  S4 & S5  */
#define TRM_R5                        4      /*  S6 & S7  */
#define TRM_R6                        5      /*  S8  */
#define TRM_R7                        6      /*  S9  */
#define TRM_R8                        7      /*  S10  */
#define TRM_TOTAL_REGIONS             8

/*  OROPHARYNX SECTIONS  */
#define TRM_S1                        0      /*  R1  */
#define TRM_S2                        1      /*  R2  */
#define TRM_S3                        2      /*  R3  */
#define TRM_S4                        3      /*  R4  */
#define TRM_S5                        4      /*  R4  */
#define TRM_S6                        5      /*  R5  */
#define TRM_S7                        6      /*  R5  */
#define TRM_S8                        7      /*  R6  */
#define TRM_S9                        8      /*  R7  */
#define TRM_S10                       9      /*  R8  */
#define TRM_TOTAL_SECTIONS            10

/*  OROPHARYNX SCATTERING JUNCTION COEFFICIENTS (BETWEEN EACH REGION)  */
#define TRM_C1                        TRM_R1     /*  R1-R2 (S1-S2)  */
#define TRM_C2                        TRM_R2     /*  R2-R3 (S2-S3)  */
#define TRM_C3                        TRM_R3     /*  R3-R4 (S3-S4)  */
#define TRM_C4                        TRM_R4     /*  R4-R5 (S5-S6)  */
#define TRM_C5                        TRM_R5     /*  R5-R6 (S7-S8)  */
#define TRM_C6                        TRM_R6     /*  R6-R7 (S8-S9)  */
#define TRM_C7                        TRM_R7     /*  R7-R8 (S9-S10)  */
#define TRM_TOTAL_COEFFICIENTS        (TRM_TOTAL_REGIONS - 1)

/*  NASAL TRACT SECTIONS  */
#define TRM_N1                        0
#define TRM_VELUM                     TRM_N1
#define TRM_N2                        1
#define TRM_N3                        2
#define TRM_N4                        3
#define TRM_N5                        4
#define TRM_N6                        5
#define TRM_N7                        6
#define TRM_TOTAL_NASAL_SECTIONS      7

/*  NASAL TRACT COEFFICIENTS  */
#define TRM_NC1                       TRM_N1     /*  N1-N2  */
#define TRM_NC2                       TRM_N2     /*  N2-N3  */
#define TRM_NC3                       TRM_N3     /*  N3-N4  */
#define TRM_NC4                       TRM_N4     /*  N4-N5  */
#define TRM_NC5                       TRM_N5     /*  N5-N6  */
#define TRM_NC6                       TRM_N6     /*  N6-N7  */
#define TRM_TOTAL_NASAL_COEFFICIENTS  (TRM_TOTAL_NASAL_SECTIONS - 1)

/*  FRICATION INJECTION COEFFICIENTS  */
#define TRM_FC1                       0      /*  S3  */
#define TRM_FC2                       1      /*  S4  */
#define TRM_FC3                       2      /*  S5  */
#define TRM_FC4                       3      /*  S6  */
#define TRM_FC5                       4      /*  S7  */
#define TRM_FC6                       5      /*  S8  */
#define TRM_FC7                       6      /*  S9  */
#define TRM_FC8                       7      /*  S10  */
#define TRM_TOTAL_FRIC_COEFFICIENTS   8

#define TUBE_MODEL_AIR_TEMPERATURE 35.0 /* deg C */

#endif /* TUBE_MODEL_CONSTANTS_H_ */
