/***************************************************************************
 *  Copyright 1991, 1992, 1993, 1994, 1995, 1996, 2001, 2002, 2004, 2007   *
 *    David R. Hill, Leonard Manzara, Craig Schock,                        *
 *    Steve Nygard                                                         *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/***************************************************************************
 * 2007-12-21  mym                                                         *
 *   This code is a translated (from C to C++) and modified version of a   *
 *   GnuSpeech's source code.                                              *
 *                                                                         *
 ***************************************************************************/

#include "util.h"



namespace Cgstmad {
namespace Util {

/******************************************************************************
 *
 *	function:	iZero
 *
 *	purpose:	Returns the value for the modified Bessel function of
 *                       the first kind, order 0, as a double.
 *
 *       arguments:      x - input argument
 *
 *	internal
 *	functions:	none
 *
 *	library
 *	functions:	none
 *
 ******************************************************************************/
double
iZero(double x)
{
	const double epsilon = 1.0e-21;

	const double halfX = x * 0.5;
	int n = 1;
	double sum = 1.0, u = 1.0;
	do {
		const double temp = halfX / n;
		u *= temp * temp;
		sum += u;
		++n;
	} while (u >= (epsilon * sum));

	return sum;
}

} /* namespace Util */
} /* namespace Cgstmad */
