/***************************************************************************
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef VIBRATO_FILTER_H_
#define VIBRATO_FILTER_H_

#include "cgstmad_trm.h"

#include <memory>

#include "sine_wavetable.h"



namespace Cgstmad {

/*******************************************************************************
 *
 */
template<typename F>
class VibratoFilter {
public:
	VibratoFilter(F sampleRate, F frequency);

	F filter(F value, F vibratoFactor);
private:
	std::auto_ptr<SineWavetable> sineWavetable_;
};



/*******************************************************************************
 * Constructor.
 */
template<typename F>
VibratoFilter<F>::VibratoFilter(F sampleRate, F frequency)
		: sineWavetable_(new SineWavetable(sampleRate, frequency, 0.5))
{
}

/*******************************************************************************
 *
 */
template<typename F>
F
VibratoFilter<F>::filter(F value, F vibratoFactor)
{
	if (vibratoFactor == 0.0) return value;

	sineWavetable_->nextPosition();
	const F factor = 1.0f + vibratoFactor * sineWavetable_->get();
	return factor > 0.0 ? value * factor : 0.0;
}

} /* namespace Cgstmad */

#endif /*VIBRATO_FILTER_H_*/
