/***************************************************************************
 *  Copyright 2008 Marcelo Y. Matuda                                       *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef SIMPLE_XML_PARSER_H_
#define SIMPLE_XML_PARSER_H_

#ifndef USE_XERCES

#include "main.h"

#include <stack>
#include <string>
#include <tr1/unordered_map>



namespace Cgstmad {

/*******************************************************************************
 * This class is not XML-compliant. It is used to read the file artic.xml.
 *
 * Notes:
 *   - All the elements must be accessed, and in the same order they
 *     appear in the XML.
 *   - The XML is supposed to contain only "ISO-8859-1" characters.
 */
class SimpleXmlParser {
public:
	SimpleXmlParser(const std::string& filePath);
	~SimpleXmlParser();

	const std::string* getFirstChild();
	const std::string* getFirstChild(const std::string& name);

	const std::string* getNextSibling();
	const std::string* getNextSibling(const std::string& name);

	const std::string& getAttribute(const std::string& name);

	const std::string& getText();
private:
	typedef std::tr1::unordered_map<std::string, std::string> AttributeMap;

	bool closedTag_;
	std::string::size_type pos_;
	std::string source_;
	std::string text_;
	std::string tag_;
	std::string tagExtra_;
	std::stack<std::string> tagStack_;
	AttributeMap attributeMap_;

	std::string::size_type find(char c);
	std::string::size_type find(const std::string& s);
	void skipSpaces();
	bool finished() const;
	char nextChar();
	bool isInvalidPosition(std::string::size_type p) const;
	void getTagInfo(std::string::size_type size);
	void closePreviousSiblingTag();
	void closeParentTag();
	void fillAttributeMap();
};

} /* namespace Cgstmad */

#endif /* USE_XERCES */

#endif /*SIMPLE_XML_PARSER_H_*/
