/***************************************************************************
 *  Copyright 2007, 2008, 2009 Marcelo Y. Matuda                           *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef XML_EXCEPTION_H_
#define XML_EXCEPTION_H_

#include "main.h"

#include <exception>
#include <sstream>



namespace Cgstmad {

/*******************************************************************************
 * Class used to build the error message of the exception.
 */
class XmlError {
public:
	XmlError(const char* message = 0);

	template<typename T> XmlError& operator<<(const T& complement) throw();
	const char* constructorMessage() const { return constructorMessage_; }
	const std::ostringstream& messageBuffer() const { return messageBuffer_; }
private:
	const char* constructorMessage_;
	std::ostringstream messageBuffer_;

	XmlError(const XmlError&);
	XmlError& operator=(const XmlError&);
};

template<typename T>
XmlError&
XmlError::operator<<(const T& complement) throw() {
	try {
		messageBuffer_ << complement;
	} catch (...) {
		// Ignore.
	}
	return *this;
}

/*******************************************************************************
 *
 */
class XmlException : public std::exception {
public:
	XmlException(const char* message) throw();
	XmlException(const XmlError& error) throw();
	virtual ~XmlException() throw() {}

	virtual const char* what() const throw();
private:
	enum {
		MESSAGE_BUFFER_SIZE = 2048
	};

	char message_[MESSAGE_BUFFER_SIZE];

	void setMessage(const char* message);
};

} /* namespace Cgstmad */

#endif /* XML_EXCEPTION_H_ */
