/***************************************************************************
 *  Copyright 2007, 2008 Marcelo Y. Matuda                                 *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef XML_UTIL_H_
#define XML_UTIL_H_

#include "main.h"

#ifdef USE_XERCES

#include <string>
#include <vector>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/util/TransService.hpp>

#ifdef XERCES_CPP_NAMESPACE_USE
XERCES_CPP_NAMESPACE_USE
#endif



namespace Cgstmad {

typedef std::basic_string<XMLCh> XmlString;
//TODO: Specialize char_traits for XMLCh



/*******************************************************************************
 * Utility class for XML handling.
 *
 * Notes:
 * - An instance of this class may not be used by more than one thread.
 * - XMLString::transcode() didn't work as expected.
 */
class XmlUtil {
public:
	XmlUtil();
	~XmlUtil();

	void getAttributeValue(const DOMNode* node, const char* name, std::string& outValue);
	void getText(const DOMNode* node, std::string& outText);
	void convertToXmlString(const char* from, XmlString& to);
	void convertFromXmlString(const XMLCh* from, std::string& to);

	static bool isElement(const DOMNode* node);
	static bool isText(const DOMNode* node);
	static bool isNamed(const DOMNode* node, const XMLCh* name);
	static bool isNamedElement(const DOMNode* node, const XMLCh* name);
private:
	enum { BUF_SIZE = 1024 };
	typedef std::vector<unsigned char> CharSizeVector;

	XMLTranscoder* transcoder_;
	CharSizeVector charSizes_;
	XMLByte xmlByteBuf_[BUF_SIZE];

	XmlUtil(const XmlUtil&);
	XmlUtil& operator=(const XmlUtil&);
};

/*******************************************************************************
 *
 */
class LocalXmlErrorHandler : public ErrorHandler {
public:
	LocalXmlErrorHandler();
	virtual ~LocalXmlErrorHandler();

	virtual void warning(const SAXParseException& exc);
	virtual void error(const SAXParseException& exc);
	virtual void fatalError(const SAXParseException& exc);
	virtual void resetErrors();
private:
	XmlUtil util_;
};

} /* namespace Cgstmad */

#endif /* USE_XERCES */

#endif /*XML_UTIL_H_*/
